/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: Dual Core LED Flasher for MCB4300
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2004-2014 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include "LPC54xxx.h"                   // Device header
#include "Board_LED.h"

#define M0plus_IMAGE_START    0x20000   /* address where the M0+ Image is located in the flash */

const __attribute__((at(M0plus_IMAGE_START)))
#include "CM0plus_Image.c"


volatile uint32_t ledState;             /* must be global because we use the addresss of it */
uint32_t M0plus_Trigger;


/*----------------------------------------------------------------------------
  MAILBOX functions
 *----------------------------------------------------------------------------*/
static void MAILBOX_mutexWait(void) {

  while (LPC_MAILBOX->MUTEX == 0) __NOP();      /* Wait forever for mutex     */
}

static void MAILBOX_mutexRelease(void) {

  LPC_MAILBOX->MUTEX = 1;                       /* release mutex              */
}

static void MAILBOX_Initialize(void) {

  LPC_SYSCON->AHBCLKCTRL0    |= (1ul << 26);    /* Enable clock for Mailbox   */
  LPC_SYSCON->PRESETCTRLSET0  = (1ul << 26);    /* assert reset for Mailbox   */
  LPC_SYSCON->PRESETCTRLCLR0  = (1ul << 26);    /* deassert reset for Mailbox */

  MAILBOX_mutexRelease();
}


/*----------------------------------------------------------------------------
  SysTick IRQ Handler @ 10ms
 *----------------------------------------------------------------------------*/
void SysTick_Handler (void) {
  static uint32_t ticks;
  
  if (ticks++ > 24) {
    ticks = 0;
    M0plus_Trigger = 1;
  }

}

/*----------------------------------------------------------------------------
  MAILBOX_IRQHandler
 *----------------------------------------------------------------------------*/
void MAILBOX_IRQHandler(void) {

  LPC_MAILBOX->IRQ1CLR = 0xFFFFFFFF;            /* Clear own mailbox          */

  LED_SetOut(ledState);                         /* output the LED state       */
}


/*----------------------------------------------------------------------------
  Main Program
 *----------------------------------------------------------------------------*/
int main(void) {
  volatile uint32_t stopHereForM0plusDebugging = 0;
	
  ledState = 1;                                 /* value is changed from M0+  */
  
  SystemCoreClockUpdate();
  LED_Initialize();

  /* Initialize the mailbox with initial mutex free (M4 only) */
  MAILBOX_Initialize();

  NVIC_EnableIRQ(MAILBOX_IRQn);                 /* Enable mailbox interrupt   */
  SysTick_Config(SystemCoreClock/100);          /* SysTick irq each 10 ms     */

  /* Boot M0+ core */
  /* Setup M0+ stack pointer and  reset vector ( from M0+ flash image) */
  LPC_SYSCON->CPSTACK = (*(uint32_t *)(M0plus_IMAGE_START + 0));
  LPC_SYSCON->CPBOOT  = (*(uint32_t *)(M0plus_IMAGE_START + 4));

  /* Enable M0+ clock and perform M0+ reset */
  LPC_SYSCON->CPUCTRL |= ((0xC0C4ul << 16) |
                          (     1ul <<  3)  );                     /* enable M0+ clock   */
  LPC_SYSCON->CPUCTRL |= ((0xC0C4ul << 16) |
                          (     1ul <<  5)  );                     /* assert M0+ reset   */
  LPC_SYSCON->CPUCTRL  = ((0xC0C4ul << 16) |
                          (LPC_SYSCON->CPUCTRL & ~(1ul << 5)));    /* deassert M0+ reset */

  stopHereForM0plusDebugging = 1;
  while (1) {  

    if (M0plus_Trigger) {
      M0plus_Trigger  = 0;
      
      MAILBOX_mutexWait();
      LPC_MAILBOX->IRQ0 = (uint32_t)&ledState;  /* signal address to M0+      */
      MAILBOX_mutexRelease();
    }
  }

}
